#!/usr/bin/python

import os
import sys
import socket
from optparse import OptionParser

VERSION = '1.0.0-CDP00002'

#Add libs directory to the path so we can import local libraries from there
libs_dir = os.path.dirname(os.path.abspath(__file__)) + '/libs'
sys.path.append(libs_dir)

import cdp

####################################
#Setup UDP listening socket default#
####################################
UDP_IP   = '239.255.76.67'
UDP_PORT = 7667
IFACE_IP = '127.0.0.1'


#####################################################
#Parse command line options and store appropriately #
#####################################################
parser = OptionParser(usage="usage: %prog [options]", version="%prog "+VERSION)
parser.add_option("-u", "--udp",  action="store", type="string", dest="udp_cfg",
                        help="UDP port configurations as IP:PORT ie. 239.255.76.67:7667")
parser.add_option("-i", "--iface-ip", action="store", type="string", dest="IFACE_IP",
                        help="Interface IP for VLAN")
(options, args) = parser.parse_args()
option_dict = vars(options)

if option_dict['udp_cfg'] is not None :
    [UDP_IP, UDP_PORT] = option_dict['udp_cfg'].split(':')
    UDP_PORT = int(UDP_PORT)
print "Using IP:" + UDP_IP + " Port:%d" % UDP_PORT


#################################
#Setup UDP socket for listening #
#################################
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
sock.setsockopt(socket.SOL_IP,socket.IP_ADD_MEMBERSHIP, socket.inet_aton(UDP_IP)+socket.inet_aton(IFACE_IP))
if sys.platform == 'win32' : sock.bind((IFACE_IP, UDP_PORT))
else :                       sock.bind((  UDP_IP, UDP_PORT))

##############################################################
#Attach class output print functions to the callback so they #
#print automatically when decoded                            #
##############################################################
print "Adding callback to print headers."
cdp.header.callback = cdp.header.output
for data_type in cdp.public_types + cdp.private_types :
    _exec_string = 'cdp.' + data_type + '.callback = cdp.' + data_type + '.output'
    print "Adding callback to print type : %s" %data_type
    exec( _exec_string )

################################
# Main Loop :                  #
#  Listen to socket and decode #
################################
while True:
    data, addr = sock.recvfrom(2048)

    try:
        packet = cdp.CDP(data)   #Create and decode packet using CDP protocol
    except ValueError:
        continue
