
from io import BytesIO
import struct

import cdp

##NOTE : 'eval' and 'setattr' are used in the code below to allow for dynamic naming of 
##       class attributes based on the data types existing in the data type folders.

class CDP(object) : 
    """CDP : Ciholas Data Protocol Python Class Definition"""

    #Initialize the class. If a buffer is given as a parameter decode by default
    def __init__(self, buf=[]) :

        self.header = []           #Initialize the CDP header to Empty

        for data_type in cdp.public_types + cdp.private_types :   #Loop through all types in the dtypes and pdtypes class folders
            setattr(self, data_type + '_data', [])                #Add attribute self.[type_name]_data and set to Empt

        if buf : self.decode(buf)  #Decode the buffer if given


    def decode(self, buf) :

        _size = len(buf)     #Save the buffer size
        _data = BytesIO(buf) #Store in a readable buffer type

        #Check that the packet is at least large enough to hold a CDP header
        if _size >= cdp.header.size :
            self.header = cdp.header(_data.read(cdp.header.size))  #Decode and Save the header data
            _size      -= cdp.header.size                          #Update the buffer size remaining
        else :
            raise ValueError("Packet Size Error")

        #Loop through the remaining buffer, popping off data types and decoding
        while _size > cdp.data_header.size :

            #Grab the next data header
            _data_header = cdp.data_header(_data.read(cdp.data_header.size))
            _read_size = _data_header.data_size

            ##############################################
            ##Check all known data types, decode, and save
            _data_decoded = False

            #Decode all data types available in the dtypes and pdtypes class folders
            for data_type in cdp.public_types + cdp.private_types :
                #Use getattr to lookup the default type_value for the type. 
                #If the data type of the current packet matches, create a class 
                #instance for that type and append it to the array of data for that type.
                if _data_header.data_type == getattr( getattr(cdp, data_type), 'type_value') :
                    eval( 'self.' + data_type + '_data.append(cdp.' + data_type + '(_data.read(_read_size)) )')
                    _data_decoded = True

            #Skip over un-recognized data types
            if not _data_decoded : _data.read(_read_size) #Skip over Un-Known data type

            #Update the remaining buffer size variable
            _size -= cdp.data_header.size + _data_header.data_size

