import struct

class mpu_quaternion_v1 : 
    """CDP Data Type : Ciholas Data Protocol Quaternion Data Type Definition"""
    type_value = 0x0004
    size = 20

    def __init__(self, data=[]) :
        #Initialize containers to reasonable values. This isn't necessary, but makes code readable
        self.wxyz = [0.0, 0.0, 0.0, 0.0] # 4-element array containing 4-byte raw quaternion values
        self.gnt = 0

        if data : 
            self.decode(data)
            self.callback()

    def decode(self, data) :
        _w,_x,_y,_z, \
        self.gnt = struct.unpack("<4s4s4s4sI", data)
        self.wxyz[0] = ord(_w[3])*(1<<24) + ord(_w[2])*(1<<16) + ord(_w[1])*(1<<8) + ord(_w[0])
        self.wxyz[1] = ord(_x[3])*(1<<24) + ord(_x[2])*(1<<16) + ord(_x[1])*(1<<8) + ord(_x[0])
        self.wxyz[2] = ord(_y[3])*(1<<24) + ord(_y[2])*(1<<16) + ord(_y[1])*(1<<8) + ord(_y[0])
        self.wxyz[3] = ord(_z[3])*(1<<24) + ord(_z[2])*(1<<16) + ord(_z[1])*(1<<8) + ord(_z[0])
        if self.wxyz[0] > 2**31 : self.wxyz[0] -= 2*(2**31)
        if self.wxyz[1] > 2**31 : self.wxyz[1] -= 2*(2**31)
        if self.wxyz[2] > 2**31 : self.wxyz[2] -= 2*(2**31)
        if self.wxyz[3] > 2**31 : self.wxyz[3] -= 2*(2**31)
        self.wxyz[0] = self.wxyz[0] * 1.0 / (1<<30)
        self.wxyz[1] = self.wxyz[1] * 1.0 / (1<<30)
        self.wxyz[2] = self.wxyz[2] * 1.0 / (1<<30)
        self.wxyz[3] = self.wxyz[3] * 1.0 / (1<<30)

    def callback(self) : pass

    def output(self) : 
        print "\tquat   " + \
              " %0.3f," % self.wxyz[0] + \
              " %0.3f," % self.wxyz[1] + \
              " %0.3f," % self.wxyz[2] + \
              " %0.3f"  % self.wxyz[3] + \
              " %d"     % self.gnt
              
